/*-------------------------------------------------------------------
Name......... duart.h
Author....... Marc Adams
Date......... 08/09/03
Description.. Header file for the generic 68692 DUART driver
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

#ifndef HARDWARE_DUART
#define HARDWARE_DUART

/****	Defines */

/* Mode register 1 bits */
#define	PARITYEVEN	0x00
#define	PARITYODD	0x04
#define	PARITYNONE	0x10
#define	PARITYBITS	0x1c

#define	BITS6		0x01
#define	BITS7		0x02
#define	BITS8		0x03
#define	CHARBITS	0x03

#define RXRTS		0x80
#define RXNONE		0x00
#define RXBITS		0x80

/* Mode register 2 bits */
#define	STOP1		0x07
#define	STOP2		0x0f
#define	STOPBITS	0x0f

#define	TXRTS		0x20
#define	TXCTS		0x10
#define	TXNONE		0x00
#define	TXBITS		0x30

/* Auxiliary control register */
#define	BAUDSET1	0x00
#define	BAUDSET2	0x80
#define	BAUDSETBITS	0x80

/*	clock select register */
#define	RXBAUDBITS	0xf0
#define	TXBAUDBITS	0x0f

/****	Command register */

#define	TX_DISABLE_CMD	0x08
#define	TX_ENABLE_CMD	0x04
#define	TX_RESET_CMD	0x30

#define	RX_DISABLE_CMD	0x02
#define	RX_ENABLE_CMD	0x01
#define	RX_RESET_CMD	0x20

#define	ERR_RESET_CMD	0x40
#define	MR_RESET_CMD	0x10

/*	Interrupt status register */
#define	TXRDYA			0x01
#define	RXRDYA			0x02
#define	DBA				0x04
#define	XTAL_RDY		0x08
#define	TXRDYB			0x10
#define	RXRDYB			0x20
#define	DBB				0x40
#define	COS				0x80

/*	Status register */
#define	SR_RB			0x80
#define	SR_FE			0x40
#define	SR_PE			0x20
#define	SR_OE			0x10

#define	ERROR_BITS	(SR_RB+SR_FE+SR_PE+SR_OE)

/****	Type Definitions */

/* Write Registers */
typedef	struct reg68692wr
{
	volatile	UBYTE	mra;
	volatile	UBYTE	csra;
	volatile	UBYTE	cra;
	volatile	UBYTE	tba;
	volatile	UBYTE	acr;
	volatile	UBYTE	ier;
	volatile	UBYTE	ctur;
	volatile	UBYTE	ctlr;

	volatile	UBYTE	mrb;
	volatile	UBYTE	csrb;
	volatile	UBYTE	crb;
	volatile	UBYTE	tbb;
	volatile	UBYTE	ivr;
	volatile	UBYTE	opcr;
	volatile	UBYTE	bset;
	volatile	UBYTE	bclr;
} REG68692W;

/* Read registers */
typedef	struct reg68692r
{
	volatile	UBYTE	mra;
	volatile	UBYTE	sra;
	volatile	UBYTE	brgtest;
	volatile	UBYTE	rba;
	volatile	UBYTE	ipcr;
	volatile	UBYTE	isr;
	volatile	UBYTE	ctru;
	volatile	UBYTE	ctrl;

	volatile	UBYTE	mrb;
	volatile	UBYTE	srb;
	volatile	UBYTE	csrb;
	volatile	UBYTE	rbb;
	volatile	UBYTE	ivr;
	volatile	UBYTE	ip;
	volatile	UBYTE	strctr;
	volatile	UBYTE	stpctr;
} REG68692R;

typedef	union	reg68692
{
	REG68692R	read;
	REG68692W	write;
} REG68692;

/* RAM based write registers */
typedef	struct ram68692w
{
	volatile	UBYTE	mr1a;
	volatile	UBYTE	mr2a;
	volatile	UBYTE	csra;

	volatile	UBYTE	mr1b;
	volatile	UBYTE	mr2b;
	volatile	UBYTE	csrb;

	volatile	UBYTE	acr;
	volatile	UBYTE	ier;
	volatile	UBYTE	ctur;
	volatile	UBYTE	ctlr;
} RAM68692W;

typedef	struct	duartcfg
{
	void*		base;
	void		(*isr)();
	BYTE*		txabuffer;
	BYTE*		rxabuffer;
	BYTE*		txbbuffer;
	BYTE*		rxbbuffer;
	BYTE		txamask;
	BYTE		rxamask;
	BYTE		txbmask;
	BYTE		rxbmask;

	BYTE		txbauda;
	BYTE		rxbauda;
	BYTE		bitsa;
	BYTE		stopa;
	BYTE		paritya;
	BYTE		txhsa;
	BYTE		rxhsa;

	BYTE		txbaudb;
	BYTE		rxbaudb;
	BYTE		bitsb;
	BYTE		stopb;
	BYTE		parityb;
	BYTE		txhsb;
	BYTE		rxhsb;
} DUARTCFG;


typedef	struct	buffer
{
	UBYTE	txin;
	UBYTE	txout;
	UBYTE	txmask;
	UBYTE	rxin;
	UBYTE	rxout;
	UBYTE	rxmask;
	UBYTE*	txbuffer;
	UBYTE*	rxbuffer;
} SERIALBUFFER;

typedef	struct	duartvars
{
	const DUARTCFG*	config;
	REG68692*		base;
	RAM68692W		regs;
	SERIALBUFFER	cha;
	SERIALBUFFER	chb;
} DUARTVARS;


typedef	struct duartservices
{
	BYTE	(*configure)(const DEVICE* dev,BYTE txbaud,BYTE rxbaud,BYTE data,BYTE stop,BYTE parity,BYTE txhs ,BYTE rxhs);
	BYTE	(*tx)(const DEVICE* dev,char ch);
	WORD	(*rx)();
	BYTE 	(*txempty)();
	BYTE 	(*rxempty)();
	void 	(*cleartx)();
	void 	(*clearrx)();
	void	(*setrouting)(const DEVICE* dev, BYTE channel, BYTE route);
} DUARTSERVICES;

/****	Prototypes */

#include	"duart.p"

#endif
/*************************************************************************/
